@@ -33,8 +33,6 @@ class _Bonds(_Table):
 
         self._system = None
 
-        self._bondset = self._configuration.bondset
-
         super().__init__(configuration._system_db, "bond")
 
     def __enter__(self):
@@ -54,6 +52,11 @@ class _Bonds(_Table):
         """Return a boolean if this object is equal to another"""
         raise NotImplementedError()
 
+    @property
+    def asymmetric_bondorders(self):
+        """The bond orders of the asymmetric bonds."""
+        return self.get_column_data("bondorder")
+
     @property
     def ids(self):
         """The ids of the bonds."""
@@ -62,12 +65,30 @@ class _Bonds(_Table):
     @property
     def bondorders(self):
         """The bond orders."""
-        return self.get_column_data("bondorder")
+        if self.configuration.symmetry.n_symops == 1:
+            return self.asymmetric_bondorders
+        else:
+            bondorders = self.asymmetric_bondorders
+            asym_bonds = self.configuration.symmetry.bond_to_asymmetric_bond
+
+            return [bondorders[i] for i in asym_bonds]
 
     @property
     def bondset(self):
         """The bondset for these bonds."""
-        return self._bondset
+        return self._configuration.bondset
+
+    @property
+    def bonds_for_asymmetric_bonds(self):
+        """List of bonds for each asymmetric bond."""
+        if self.configuration.symmetry.n_symops == 1:
+            result = [[i] for i in range(self.n_asymmetric_bonds)]
+        else:
+            result = [[] for i in range(self.n_asymmetric_bonds)]
+            to_asym = self.configuration.symmetry.bond_to_asymmetric_bond
+            for i, asym_bond in enumerate(to_asym):
+                result[asym_bond].append(i)
+        return result
 
     @property
     def configuration(self):
@@ -85,13 +106,34 @@ class _Bonds(_Table):
         return self.system_db.db
 
     @property
-    def n_bonds(self):
-        """The number of bonds."""
+    def loglevel(self):
+        """The logging level for this module."""
+        result = logger.getEffectiveLevel()
+        tmp = logging.getLevelName(result)
+        if "Level" not in tmp:
+            result = tmp
+        return result
+
+    @loglevel.setter
+    def loglevel(self, value):
+        logger.setLevel(value)
+
+    @property
+    def n_asymmetric_bonds(self):
+        """The number of asymmetric bonds."""
         self.cursor.execute(
             "SELECT COUNT(*) FROM bondset_bond WHERE bondset = ?", (self.bondset,)
         )
         return self.cursor.fetchone()[0]
 
+    @property
+    def n_bonds(self):
+        """The number of bonds."""
+        if self.configuration.symmetry.n_symops == 1:
+            return self.n_asymmetric_bonds
+        else:
+            return len(self.configuration.symmetry.bond_atoms)
+
     @property
     def system_db(self):
         """The system database that we belong to."""
@@ -191,16 +233,30 @@ class _Bonds(_Table):
 
         i2 = []
         j2 = []
-        for i_, j_ in zip(i, j):
-            # will need to handle offsets here at some point
+        symop1 = []
+        symop2 = []
+
+        op1s = kwargs["symop1"] if "symop1" in kwargs else [0] * len(i)
+        op2s = kwargs["symop2"] if "symop2" in kwargs else [0] * len(i)
+
+        if isinstance(op1s, int):
+            op1s = [op1s]
+        if isinstance(op2s, int):
+            op2s = [op2s]
+
+        for i_, j_, op1, op2 in zip(i, j, op1s, op2s):
             if not isinstance(i_, int) or not isinstance(j_, int):
                 raise TypeError("'i' and 'j', the atom indices, must be integers")
             if i_ < j_:
                 i2.append(i_)
                 j2.append(j_)
+                symop1.append(op1)
+                symop2.append(op2)
             else:
                 i2.append(j_)
                 j2.append(i_)
+                symop1.append(op2)
+                symop2.append(op1)
 
         ids = super().append(i=i2, j=j2, **kwargs)
 
@@ -208,15 +264,19 @@ class _Bonds(_Table):
         table = _Table(self.system_db, "bondset_bond")
         table.append(bondset=self.bondset, bond=ids)
 
+        self.configuration.symmetry.reset_bonds()
+
         return ids
 
-    def bonds(self, *args):
+    def bonds(self, *args, asymmetric=False):
         """Returns an iterator over the bonds.
 
         Parameters
         ----------
         args : [str]
             Added selection criteria for the SQL, one word at a time.
+        asymmetric : bool
+            Whether to produce only the asymmetric bonds
 
         Returns
         -------
@@ -276,6 +336,7 @@ class _Bonds(_Table):
                 raise ValueError("The bond did not exist.")
         else:
             self.cursor.execute("DELETE FROM bond WHERE id = ?", (bond_id,))
+            self.configuration.symmetry.reset_bonds()
 
     def delete(self, atoms=None):
         """Deletes all the bonds, optionally from atoms.
@@ -644,6 +705,202 @@ class _Bonds(_Table):
         table = _Table(self.system_db, "bondset_bond")
         table.append(bondset=self.bondset, bond=ids)
 
+    def get_lengths(self, asymmetric=False, as_array=False):
+        """Return the lengths of the bonds.
+
+        Parameters
+        ----------
+        asymmetric : bool = False
+            Return the lengths of just the asymmetric bonds. Default, all bonds
+        as_array : bool = False
+            Return the lengths as a Numpy array. Defaults to Python list
+        """
+        if self.configuration.periodicity == 0:
+            xyz = self.configuration.atoms.get_coordinates(as_array=True)
+            if self.configuration.symmetry.n_symops > 1:
+                raise NotImplementedError("Symmetry not implemented for molecules.")
+            atom_pairs = self.configuration.symmetry.bond_atoms
+            Is = [i for i, j in atom_pairs]
+            Js = [j for i, j in atom_pairs]
+            R = np.linalg.norm(xyz[Is] - xyz[Js], axis=1)
+        else:
+            atom_pairs = self.configuration.symmetry.bond_atoms
+
+            Is = [i for i, j in atom_pairs]
+            Js = [j for i, j in atom_pairs]
+
+            uvw = self.configuration.atoms.get_coordinates(as_array=True)
+            offsets = self.configuration.symmetry.bond_offsets
+
+            offsets1 = [off[0] for off in offsets]
+            offsets2 = [off[1] for off in offsets]
+            delta = uvw[Js] + offsets2 - uvw[Is] - offsets1
+            dxyz = self.configuration.cell.to_cartesians(delta, as_array=True)
+            R = np.linalg.norm(dxyz, axis=1)
+
+            if logger.isEnabledFor(logging.DEBUG):
+                import mendeleev
+
+                uvw = self.configuration.atoms.get_coordinates(as_array=True)
+                logger.debug(
+                    "i-j |    u     v     w    o1   o2   o3     "
+                    "u     v     w    o1   o2   o3   |   du    dv     dw |   r"
+                )
+                for ij, off, i, j, r in zip(atom_pairs, offsets, Is, Js, R.tolist()):
+                    off1x, off1y, off1z = off[0]
+                    off2x, off2y, off2z = off[1]
+                    xi, yi, zi = uvw[i].tolist()
+                    xj, yj, zj = uvw[j].tolist()
+                    delta = uvw[j] - uvw[i]
+                    xd, yd, zd = delta.tolist()
+                    logger.debug(
+                        f"{i}-{j} | "
+                        f"{xi:5.2f} {yi:5.2f} {zi:5.2f}({off1x:4} {off1y:4} {off1z:4}) "
+                        f"{xj:5.2f} {yj:5.2f} {zj:5.2f}({off2x:4} {off2y:4} {off2z:4}) "
+                        f" | {xd:5.2f} {yd:5.2f} {zd:5.2f}"
+                        f" |  {r=:7.4f}"
+                    )
+
+                generators = self.configuration.symmetry.atom_generators
+                gens = []
+                for tmp in generators:
+                    gens.extend(tmp)
+                to_asym = self.configuration.symmetry.atom_to_asymmetric_atom
+                logger.debug("")
+                logger.debug("Coordinates")
+                symbols = self.configuration.atoms.symbols
+                els = mendeleev.element(symbols)
+                radii = [el.covalent_radius_pyykko / 100.0 for el in els]
+
+                for symbol, radius, tmp in zip(symbols, radii, uvw.tolist()):
+                    logger.debug(
+                        f"    {symbol:2s} {radius=:7.4f} {tmp[0]:7.4f} {tmp[1]:7.4f} "
+                        f"{tmp[2]:7.4f}"
+                    )
+                logger.debug("")
+                logger.debug("Distance matrix")
+                count = 0
+                n_atoms = self.configuration.n_atoms
+                found = []
+                for i in range(n_atoms):
+                    for j in range(n_atoms):
+                        i_asym = to_asym[i]
+                        j_asym = to_asym[j]
+                        radius_i = radii[i]
+                        radius_j = radii[j]
+                        delta = uvw[j] - uvw[i]
+                        offset = []
+                        across_cell = False
+                        cell_offset = []
+                        for x in delta.round(2).tolist():
+                            if x == -0.5:
+                                across_cell = True
+                                cell_offset.append(1)
+                            elif x == 0.5:
+                                across_cell = True
+                                cell_offset.append(-1)
+                            else:
+                                cell_offset.append(0)
+
+                            if x < -0.5:
+                                offset.append(int(0.5 - x))
+                            elif x > 0.5:
+                                offset.append(-int(0.5 + x))
+                            else:
+                                offset.append(0)
+                        # offset = np.select([tmp <= -0.5, tmp > 0.5], [1, -1], 0)
+                        delta3 = delta + offset
+                        if self.configuration.periodicity != 0:
+                            delta3 = self.configuration.cell.to_cartesians(
+                                delta3, as_array=True
+                            )
+                        r = np.round(np.linalg.norm(delta3), 4)
+                        if r > 0.0 and r < 1.1 * (radius_i + radius_j):
+                            o1, o2, o3 = offset
+                            if i > j:
+                                i_asym, j_asym = j_asym, i_asym
+                                ii, jj = j, i
+                                radius_i, radius_j = radius_j, radius_i
+                                o1, o2, o3 = -o1, -o2, -o3
+                                delta = -delta
+                            else:
+                                ii, jj = i, j
+
+                            if (ii, jj, o1, o2, o3) in found:
+                                continue
+                            found.append((ii, jj, o1, o2, o3))
+                            count += 1
+                            xi, yi, zi = uvw[ii].tolist()
+                            xj, yj, zj = uvw[jj].tolist()
+                            xd, yd, zd = delta.tolist()
+                            logger.debug(
+                                f"{count:2}: {ii}-{jj} "
+                                f"({i_asym}:{gens[ii]:2} {j_asym}:{gens[jj]:2}) {r} "
+                                f"({o1:2} {o2:2} {o3:2})"
+                                f" {xj:7.4f} {yj:7.4f} {zj:7.4f} -"
+                                f" {xi:7.4f} {yi:7.4f} {zi:7.4f} ="
+                                f" {xd:7.4f} {yd:7.4f} {zd:7.4f}"
+                            )
+                            if across_cell:
+                                # Which direction to move
+                                c1, c2, c3 = cell_offset
+                                o1, o2, o3 = o1 + c1, o2 + c2, o2 + c3
+                                xj, yj, zj = xj + c1, yj + c2, zj + c3
+                                xd, yd, zd = xd + c1, yd + c2, zd + c3
+                                count += 1
+                                logger.debug(
+                                    f"{count:2}: {ii}-{jj} "
+                                    f"({i_asym}:{gens[ii]:2} {j_asym}:{gens[jj]:2}) {r}"
+                                    f" ({o1:2} {o2:2} {o3:2})"
+                                    f" {xj:7.4f} {yj:7.4f} {zj:7.4f} -"
+                                    f" {xi:7.4f} {yi:7.4f} {zi:7.4f} ="
+                                    f" {xd:7.4f} {yd:7.4f} {zd:7.4f}"
+                                )
+
+        if asymmetric:
+            # Lazy approach ... just pull out the first bond of each group
+            rs = []
+            for tmp in self.configuration.bonds.bonds_for_asymmetric_bonds:
+                if len(tmp) == 0:
+                    rs.append(0.0)
+                else:
+                    rs.append(R[tmp[0]])
+            if as_array:
+                return np.array(rs)
+            else:
+                return rs
+        if as_array:
+            return R
+        else:
+            return R.tolist()
+
+    def symmetric_bonds_to_dataframe(self):
+        """Return the symmetric bonds as a Pandas Dataframe."""
+        symmetry = self.configuration.symmetry
+        to_asym_bond = symmetry.bond_to_asymmetric_bond
+        atoms = symmetry.bond_atoms
+        bondorders = self.asymmetric_bondorders
+        if self.configuration.periodicity == 0:
+            data = {
+                "AsymBond": [i for i in to_asym_bond],
+                "i_atom": [i for i, j in atoms],
+                "j_atom": [j for i, j in atoms],
+                "bondorder": [bondorders[i] for i in to_asym_bond],
+            }
+        else:
+            offsets = symmetry.bond_offsets
+            data = {
+                "AsymBond": [i for i in to_asym_bond],
+                "i_atom": [i for i, j in atoms],
+                "j_atom": [j for i, j in atoms],
+                "bondorder": [bondorders[i] for i in to_asym_bond],
+                "offset1": [i for i, j, k in offsets],
+                "offset2": [j for i, j, k in offsets],
+                "offset3": [k for i, j, k in offsets],
+            }
+        df = pandas.DataFrame.from_dict(data)
+        return df
+
     def to_dataframe(self):
         """Return the bonds as a Pandas Dataframe."""
         cursor = self.bonds()
@@ -677,8 +934,8 @@ class _SubsetBonds(_Bonds):
         return self.get_bond_ids()
 
     @property
-    def n_bonds(self):
-        """The number of bonds."""
+    def n_asymmetric_bonds(self):
+        """The number of asymmetric bonds."""
         sql = """
         SELECT COUNT(*) FROM bond
          WHERE id IN (SELECT bond FROM bondset_bond WHERE bondset = ?)
@@ -688,6 +945,20 @@ class _SubsetBonds(_Bonds):
         self.cursor.execute(sql, (self.bondset, self.subset_id))
         return self.cursor.fetchone()[0]
 
+    @property
+    def n_bonds(self):
+        """The number of bonds in this subset."""
+        if self.configuration.symmetry.n_symops == 1:
+            return self.n_asymmetric_bonds
+        else:
+            n = 0
+            asym_bonds = self.bonds_for_asymmetric_bonds
+            b_ids = self.configuration.bonds.ids
+            for i in self.ids:
+                index = b_ids.index(i)
+                n += len(asym_bonds[index])
+            return n
+
     def append(self, **kwargs):
         """Append one or more bonds.
 
