@@ -17,12 +17,10 @@ Bond Orders
 import io
 import json
 import logging
-import math
+import re
 
 import CifFile
 
-from .symmetry import _Symmetry
-
 logger = logging.getLogger(__name__)
 
 bond_order = {
@@ -75,14 +73,14 @@ class CIFMixin:
         with open(path, "r") as fd:
             for line in fd:
                 if line[0:5] == "data_":
-                    self.logger.debug(f"Found block {line}")
+                    logger.debug(f"Found block {line}")
                     if not in_block:
                         in_block = True
                     else:
                         new_systems, new_configurations = self.from_mmcif_text(
                             "\n".join(lines)
                         )
-                        self.logger.debug(
+                        logger.debug(
                             f"   added system {sys_db.n_systems}: {block_name}"
                         )
                         systems.extend(new_systems)
@@ -94,7 +92,7 @@ class CIFMixin:
             if len(lines) > 0:
                 # The last block just ends at the end of the file
                 new_systems, new_configurations = self.from_mmcif_text("\n".join(lines))
-                self.logger.debug(f"   added system {sys_db.n_systems}: {block_name}")
+                logger.debug(f"   added system {sys_db.n_systems}: {block_name}")
                 systems.extend(new_systems)
                 configurations.update(new_configurations)
 
@@ -116,9 +114,15 @@ class CIFMixin:
         # Get the chemical formula
         formula, empirical_formula, Z = self.formula
 
-        _id = empirical_formula.replace(" ", "")
+        system_name = self.system.name
+        conf_name = self.name
+        if system_name == "" or conf_name == "":
+            _id = empirical_formula.replace(" ", "")
+        else:
+            _id = f"SEAMM:{system_name}/{conf_name}"
+            _id = _id.replace(" ", "")
 
-        # And created the file, line-by-line
+        # And create the file, line-by-line
         lines = []
         lines.append("# Generated by MolSSI SEAMM")
         lines.append(f"data_{_id}")
@@ -126,24 +130,39 @@ class CIFMixin:
         # Cell information
         if self.periodicity == 3:
             cell = self.cell
+            symmetry = self.symmetry
             a, b, c, alpha, beta, gamma = cell.parameters
             volume = cell.volume
-            lines.append("_symmetry_space_group_name_H-M   'P 1'")
+            spgname = symmetry.group
+            if spgname == "":
+                pass
+            elif symmetry.n_symops == 1:
+                lines.append("_space_group_name_H-M_full   'P 1'")
+            else:
+                spgname_system = symmetry.spacegroup_names_to_system()[spgname]
+                lines.append(f"_space_group_{spgname_system}       '{spgname}'")
+                hall = symmetry.hall_symbol
+                lines.append(f"_symmetry_space_group_name_Hall '{hall}'")
+                it_number = symmetry.IT_number
+                lines.append(f"_space_group_IT_number          {it_number}")
             lines.append(f"_cell_length_a   {a}")
             lines.append(f"_cell_length_b   {b}")
             lines.append(f"_cell_length_c   {c}")
             lines.append(f"_cell_angle_alpha   {alpha}")
             lines.append(f"_cell_angle_beta    {beta}")
             lines.append(f"_cell_angle_gamma   {gamma}")
-            lines.append("_symmetry_Int_Tables_number   1")
             lines.append(f"_cell_volume   {volume}")
             lines.append(f"_cell_formula_units_Z   {Z}")
             lines.append("loop_")
             lines.append(" _symmetry_equiv_pos_site_id")
             lines.append(" _symmetry_equiv_pos_as_xyz")
-            lines.append("  1  'x, y, z'")
+            if symmetry.n_symops == 1:
+                lines.append("  1  x,y,z")
+            else:
+                for i, op in enumerate(symmetry.symops, start=1):
+                    lines.append(f" {i:2} {op}")
 
-        lines.append(f"_chemical_formula_structural   {empirical_formula}")
+        lines.append(f"_chemical_formula_structural   '{empirical_formula}'")
         lines.append(f"_chemical_formula_sum   '{formula}'")
 
         # The atoms
@@ -160,7 +179,7 @@ class CIFMixin:
         if "names" in atoms:
             original_names = atoms.get_column("names")
         else:
-            original_names = atoms.symbols
+            original_names = atoms.asymmetric_symbols
 
         names = []
         tmp = {}
@@ -170,16 +189,45 @@ class CIFMixin:
                 names.append(name + str(tmp[name]))
             else:
                 tmp[name] = 1
-                names.append(name)
+                names.append(name + "1")
 
-        UVW = atoms.get_coordinates(fractionals=True, in_cell="molecule")
+        if symmetry.n_symops == 1:
+            UVW = atoms.get_coordinates(
+                fractionals=True, in_cell="molecule", asymmetric=True
+            )
+        else:
+            # For the moment can't center molecules in cell with symmetry.
+            UVW = atoms.get_coordinates(fractionals=True, asymmetric=True)
 
-        symbols = atoms.symbols
+        symbols = atoms.asymmetric_symbols
         for element, name, uvw in zip(symbols, names, UVW):
             u, v, w = uvw
-            lines.append(f"{element} {name}  1  {u:.3f} {v:.3f} {w:.3f}  1")
+            lines.append(f"  {element} {name}  1  {u:.3f} {v:.3f} {w:.3f}  1")
+
+        # Handle bonds.
+        if self.n_asymmetric_bonds > 0:
+            bonds = self.bonds
+            sym_bonds = bonds.bonds_for_asymmetric_bonds
+            indx = {j: i for i, j in enumerate(atoms.ids)}
+            Is = [indx[i] for i in bonds.get_column_data("i")]
+            Js = [indx[j] for j in bonds.get_column_data("j")]
+            op1s = bonds.get_column_data("symop1")
+            op2s = bonds.get_column_data("symop2")
+            Rs = bonds.get_lengths(asymmetric=True)
+            lines.append("loop_")
+            lines.append("_geom_bond_atom_site_label_1")
+            lines.append("_geom_bond_atom_site_label_2")
+            lines.append("_geom_bond_distance")
+            lines.append("_geom_bond_site_symmetry_1")
+            lines.append("_geom_bond_site_symmetry_2")
+            for i, j, r, op1, op2, n_sym_bonds in zip(
+                Is, Js, Rs, op1s, op2s, [len(x) for x in sym_bonds]
+            ):
+                if n_sym_bonds > 0:
+                    lines.append(f"  {names[i]} {names[j]} {r:.4f} {op1} {op2}")
 
         # And that is it!
+        lines.append("")
         return "\n".join(lines)
 
     def from_cif_text(self, text):
@@ -206,7 +254,6 @@ class CIFMixin:
             )
         data_block = cif[data_blocks[0]]
 
-        # print(json.dumps({**data_block}, indent=4, sort_keys=True))
         logger.debug(json.dumps({**data_block}, indent=4, sort_keys=True))
 
         # Reset the system
@@ -227,29 +274,38 @@ class CIFMixin:
             self.coordinate_system = "fractional"
             self.cell.parameters = (a, b, c, alpha, beta, gamma)
 
-            # Add the atoms
-            # TEMPORARILY lower the symmetry to P1
-            delta = 1.0e-04
-
             # Where is the symmetry info?
-            spgname = None
+            used_symops = False
             if "_space_group_symop" + dot + "operation_xyz" in data_block:
                 symdata = "_space_group_symop" + dot + "operation_xyz"
-                operators = data_block[symdata]
+                self.symmetry.symops = data_block[symdata]
+                used_symops = True
             elif "_symmetry_equiv" + dot + "pos_as_xyz" in data_block:
                 symdata = "_symmetry_equiv" + dot + "pos_as_xyz"
-                operators = data_block[symdata]
-            elif "_symmetry" + dot + "space_group_name_H-M" in data_block:
-                spgname = data_block["_symmetry" + dot + "space_group_name_H-M"]
-                operators = _Symmetry.symops_as_strings(spgname)
+                self.symmetry.symops = data_block[symdata]
+                used_symops = True
             else:
-                raise RuntimeError(
-                    "CIF file does not contain required symmetry information. Neither "
-                    "'_symmetry_equiv" + dot + "pos_as_xyz' or "
-                    "'_space_group_symop" + dot + "operation_xyz' or "
-                    "_symmetry" + dot + "space_group_name_H-M "
-                    "is present."
-                )
+                found = False
+                for section in (
+                    "_space_group" + dot + "name_Hall",
+                    "_space_group" + dot + "name_H-M_full",
+                    "_space_group" + dot + "name_H-M_alt",
+                    "_symmetry" + dot + "space_group_name_Hall",
+                    "_symmetry" + dot + "space_group_name_H-M",
+                ):
+                    if section in data_block:
+                        found = True
+                        self.symmetry.group = data_block[section]
+                if not found:
+                    raise RuntimeError(
+                        "CIF file does not contain required symmetry information. "
+                        "Neither "
+                        "'_symmetry_equiv" + dot + "pos_as_xyz' or "
+                        "'_space_group_symop" + dot + "operation_xyz' or "
+                        "_symmetry" + dot + "space_group_name_Hall  or "
+                        "_symmetry" + dot + "space_group_name_H-M "
+                        "is present."
+                    )
 
             x_label = "_atom_site" + dot + "fract_x"
             if x_label in data_block:
@@ -357,6 +413,7 @@ class CIFMixin:
         ys = []
         zs = []
         symbols = []
+        names = []
         # May have type symbols or labels, or both. Use type symbols by preference
         if "_atom_site" + dot + "type_symbol" in data_block:
             type_section = "_atom_site" + dot + "type_symbol"
@@ -366,15 +423,24 @@ class CIFMixin:
             raise KeyError(
                 f"Neither _atom_site{dot}type_label or _atom_site{dot}label are in file"
             )
-        tmp_count = 0
-        for x, y, z, symbol, label, alt in zip(
+
+        # May need site labels later
+        if "_atom_site" + dot + "label" in data_block:
+            label_section = "_atom_site" + dot + "label"
+            have_names = True
+        else:
+            label_section = type_section
+            have_names = False
+
+        for x, y, z, symbol, label, alt, name in zip(
             data_block[x_label],
             data_block[y_label],
             data_block[z_label],
             data_block[type_section],
             labels,
             alt_ids,
-        ):  # yapf: disable
+            data_block[label_section],
+        ):
             # Atom symbols may be followed by number, etc.
             if alt != "." and label in use_alt_id and alt != use_alt_id[label]:
                 logger.debug(f"   Skipping alternate configuration {label} -- {alt}")
@@ -387,69 +453,139 @@ class CIFMixin:
                     symbol = symbol[0]
             if symbol == "D":
                 symbol = "H"
-            # These variables *are* used in the eval below.
-            x = float(x.split("(")[0])
-            y = float(y.split("(")[0])
-            z = float(z.split("(")[0])
+
+            # May have uncertainties in ()
+            x = x.split("(")[0]
+            y = y.split("(")[0]
+            z = z.split("(")[0]
+
+            # Check for 1/3 and 2/3
+            if "." in x:
+                xx, dx = x.split(".")
+                if re.fullmatch(r"333+", dx):
+                    x = float(xx) - 1.0 / 3.0 if "-" in xx else float(xx) + 1.0 / 3.0
+                elif re.fullmatch(r"66+(6|7)?", dx):
+                    x = float(xx) - 2.0 / 3.0 if "-" in xx else float(xx) + 2.0 / 3.0
+                else:
+                    x = float(x)
+            else:
+                x = float(x)
+            if "." in y:
+                yy, dy = y.split(".")
+                if re.fullmatch(r"333+", dy):
+                    y = float(yy) - 1.0 / 3.0 if "-" in yy else float(yy) + 1.0 / 3.0
+                elif re.fullmatch(r"66+(6|7)?", dy):
+                    y = float(yy) - 2.0 / 3.0 if "-" in yy else float(yy) + 2.0 / 3.0
+                else:
+                    y = float(y)
+            else:
+                y = float(y)
+            if "." in z:
+                zz, dz = z.split(".")
+                if re.fullmatch(r"333+", dz):
+                    z = float(zz) - 1.0 / 3.0 if "-" in zz else float(zz) + 1.0 / 3.0
+                elif re.fullmatch(r"66+(6|7)?", dz):
+                    z = float(zz) - 2.0 / 3.0 if "-" in zz else float(zz) + 2.0 / 3.0
+                else:
+                    z = float(z)
+            else:
+                z = float(z)
+
             logger.debug(f"xyz = {x:7.3f} {y:7.3f} {z:7.3f}")
             if self.periodicity == 3:
                 if not have_fractionals:
                     # Convert Cartesians to fractionals
                     x, y, z = self.cell.to_fractionals([[x, y, z]])[0]
+            # Non-periodic
+            xs.append(x)
+            ys.append(y)
+            zs.append(z)
+            symbols.append(symbol)
+            names.append(name)
 
-                if len(operators) > 1:
-                    # Only do this if there are symmetry operators!
-                    for symop in operators:
-                        x_eq, y_eq, z_eq = symop.split(",")
-                        x_new = eval(x_eq)
-                        y_new = eval(y_eq)
-                        z_new = eval(z_eq)
-                        # Translate into cell.
-                        x_new = x_new - math.floor(x_new)
-                        y_new = y_new - math.floor(y_new)
-                        z_new = z_new - math.floor(z_new)
-                        # check for almost 1, should be 0
-                        if abs(1 - x_new) < delta:
-                            x_new = 0.0
-                        if abs(1 - y_new) < delta:
-                            y_new = 0.0
-                        if abs(1 - z_new) < delta:
-                            z_new = 0.0
-                        found = False
-                        logger.debug(f"-->   {x_new:7.3f} {y_new:7.3f} {z_new:7.3f}")
-                        for x0, y0, z0 in zip(xs, ys, zs):
-                            if (
-                                abs(x_new - x0) < delta
-                                and abs(y_new - y0) < delta
-                                and abs(z_new - z0) < delta
-                            ):
-                                found = True
-                                logger.debug("         found!")
-                                break
-                        if not found:
-                            xs.append(x_new)
-                            ys.append(y_new)
-                            zs.append(z_new)
-                            symbols.append(symbol)
-                            tmp_count += 1
-                            # logger.info(f"{tmp_count:4} {name} {comp} {label}")
+        if have_names:
+            if "name" not in self.atoms:
+                self.atoms.add_attribute("name")
+            ids = self.atoms.append(x=xs, y=ys, z=zs, symbol=symbols, name=names)
+        else:
+            ids = self.atoms.append(x=xs, y=ys, z=zs, symbol=symbols)
+
+        # Are there bonds?
+        have_bonds = False
+        if "_geom_bond" + dot + "atom_site_label_1" in data_block:
+            label_1_section = "_geom_bond" + dot + "atom_site_label_1"
+            label_2_section = "_geom_bond" + dot + "atom_site_label_2"
+            symmetry_1_section = "_geom_bond" + dot + "site_symmetry_1"
+            symmetry_2_section = "_geom_bond" + dot + "site_symmetry_2"
+            have_bonds = True
+        elif "_geom_bond" + dot + "atom_site_id_1" in data_block:
+            label_1_section = "_geom_bond" + dot + "atom_site_id_1"
+            label_2_section = "_geom_bond" + dot + "atom_site_id_2"
+            symmetry_1_section = "_geom_bond" + dot + "site_symmetry_1"
+            symmetry_2_section = "_geom_bond" + dot + "site_symmetry_2"
+            have_bonds = True
+
+        if have_bonds:
+            if not have_names:
+                raise RuntimeError("CIF file has bonds but no atom labels (names)")
+            atom_id = {name: i for name, i in zip(names, ids)}
+            for key in (symmetry_1_section, symmetry_2_section):
+                if key not in data_block:
+                    data_block[key] = ["."] * len(data_block[label_1_section])
+
+            Is = []
+            Js = []
+            symop1s = []
+            symop2s = []
+
+            for label1, label2, sym1, sym2 in zip(
+                data_block[label_1_section],
+                data_block[label_2_section],
+                data_block[symmetry_1_section],
+                data_block[symmetry_2_section],
+            ):
+                i = atom_id[label1]
+                j = atom_id[label2]
+
+                if i < j:
+                    Is.append(i)
+                    Js.append(j)
+                    symop1s.append(sym1)
+                    symop2s.append(sym2)
                 else:
-                    xs.append(x)
-                    ys.append(y)
-                    zs.append(z)
-                    symbols.append(symbol)
-                    tmp_count += 1
-            else:
-                xs.append(x)
-                ys.append(y)
-                zs.append(z)
-                symbols.append(symbol)
-                tmp_count += 1
-        self.atoms.append(x=xs, y=ys, z=zs, symbol=symbols)
+                    Is.append(j)
+                    Js.append(i)
+                    symop1s.append(sym2)
+                    symop2s.append(sym1)
 
-        if self.periodicity == 3:
-            # Find the symmetry and set to the conventional cell.
-            self.symmetrize(symprec=0.0001)
+            self.bonds.append(
+                i=Is,
+                j=Js,
+                symop1=symop1s,
+                symop2=symop2s,
+            )
+
+        # If used symops, need to find the spacegroup name(s)
+        if used_symops:
+            before = self.symmetry.symops
+            international_name = self.symmetry.find_spacegroup_from_operators()
+            self.symmetry.update_group(international_name)
+            after = self.symmetry.symops
+            if before != after:
+                print("!!!!!!!!!!!!!!!!!!!! SYMOPS CHANGED !!!!!!!!!!!!!!!!!!!!!")
+
+        # import pprint
+
+        # print("Reading CIF file")
+        # print(f"{self.symmetry.hall_number=}")
+        # print(f"{self.symmetry.hall_symbol=}")
+        # print(f"{self.symmetry.group=}")
+        # pprint.pprint(self.atoms.get_coordinates(asymmetric=True))
+        # print("all atoms")
+        # pprint.pprint(self.atoms.get_coordinates(asymmetric=False))
+
+        # print("\n\nSymmetry info")
+        # pprint.pprint(self.get_symmetry_data(self.symmetry.hall_number))
 
         return result
 
@@ -564,6 +700,7 @@ class CIFMixin:
                 lines.append(f"MOL1 {nm1} {nm2} {order}")
 
         # And that is it!
+        lines.append("")
         return "\n".join(lines)
 
     def from_mmcif_text(self, text):
@@ -773,7 +910,7 @@ class CIFMixin:
                 model_args = {k: v[first:last] for k, v in kwargs.items()}
                 atom_ids = atoms.append(**model_args)
                 if model != n_ensemble:
-                    self.logger.warning(
+                    logger.warning(
                         f"The actual number of models ({model}) does not "
                         f"match the claimed number ({n_ensemble})."
                     )
