@@ -4,6 +4,7 @@ from collections import Counter
 from functools import reduce
 import logging
 import math
+import pprint  # noqa: E401
 
 import seekpath
 import spglib
@@ -11,23 +12,40 @@ import spglib
 from .atoms import _Atoms
 from .bonds import _Bonds
 from .cell import _Cell
+from .configuration_properties import _ConfigurationProperties
+from .subsets import _Subsets
+from .symmetry import _Symmetry
+from .table import _Table
+
+from .align import AlignMixin
 from .cif import CIFMixin
 from .cms_schema import CMSSchemaMixin
-from .configuration_properties import _ConfigurationProperties
 from .inchi import InChIMixin
 from .molfile import MolFileMixin
 from .openbabel import OpenBabelMixin
+from .openeye import OpenEyeMixin
+from .pubchem import PubChemMixin
 from .rdkit_ import RDKitMixin
 from .pdb import PDBMixin
 from .qcschema import QCSchemaMixin
 from .smiles import SMILESMixin
-from .subsets import _Subsets
-from .symmetry import _Symmetry
-from .table import _Table
 from .topology import TopologyMixin
 
 logger = logging.getLogger(__name__)
 
+spin_states = (
+    "singlet",
+    "doublet",
+    "triplet",
+    "quartet",
+    "quintet",
+    "sextet",
+    "septet",
+    "octet",
+    "nonet",
+    "decet",
+)
+
 
 class _Configuration(
     PDBMixin,
@@ -38,8 +56,11 @@ class _Configuration(
     SMILESMixin,
     TopologyMixin,
     OpenBabelMixin,
+    OpenEyeMixin,
+    PubChemMixin,
     RDKitMixin,
     QCSchemaMixin,
+    AlignMixin,
     object,
 ):
     """A configuration (conformer) of a system."""
@@ -59,6 +80,11 @@ class _Configuration(
         self._periodicity = None
         self._symmetry_id = None
 
+        self._symmetry = None  # A cache for the symmetry instance
+
+        # Initialize periodicity and symmetry to 0 and C1
+        # self.periodicity = 0
+
     def __enter__(self):
         """Copy the tables to a backup for a 'with' statement."""
         self.system_db["configuration"].__enter__()
@@ -88,6 +114,11 @@ class _Configuration(
         """The atoms for this configuration."""
         return _Atoms(self)
 
+    @property
+    def atom_to_asymmetric_atom(self):
+        """The asymmetric atom related to each atom."""
+        return self.symmetry.atom_to_asymmetric_atom
+
     @property
     def atomset(self):
         """The id of the atom set for this configuration."""
@@ -130,6 +161,11 @@ class _Configuration(
         else:
             raise RuntimeError("The atomset is already set!")
 
+    @property
+    def atom_generators(self):
+        """The symmetry operations that create the symmetric atoms."""
+        return self.symmetry.atom_generators
+
     @property
     def bonds(self):
         """The bonds for this configuration."""
@@ -331,6 +367,22 @@ class _Configuration(
             else:
                 formula.append(element)
 
+        formula = " ".join(formula)
+
+        # Handle any charge
+        if self.charge == -1:
+            formula = f"[{formula}]-"
+            counts.append(1)
+        elif self.charge == 1:
+            formula = f"[{formula}]+"
+            counts.append(1)
+        elif self.charge < 0:
+            formula = f"[{formula}]-{-self.charge}"
+            counts.append(-self.charge)
+        elif self.charge > 0:
+            formula = f"[{formula}]+{self.charge}"
+            counts.append(self.charge)
+
         # And the empirical formula
         Z = reduce(math.gcd, counts)
         empirical_formula_list = []
@@ -344,7 +396,34 @@ class _Configuration(
             else:
                 empirical_formula.append(element)
 
-        return " ".join(formula), " ".join(empirical_formula), Z
+        empirical_formula = " ".join(empirical_formula)
+
+        # Handle any charge
+        charge = self.charge / Z
+        if charge == -1:
+            empirical_formula = f"[{empirical_formula}]-"
+        elif charge == 1:
+            empirical_formula = f"[{empirical_formula}]+"
+        elif charge < 0:
+            empirical_formula = f"[{empirical_formula}]-{-charge}"
+        elif charge > 0:
+            empirical_formula = f"[{empirical_formula}]+{charge}"
+
+        return formula, empirical_formula, Z
+
+    @property
+    def group(self):
+        """The space or point group of the configuration."""
+        return self.symmetry.group
+
+    @group.setter
+    def group(self, value):
+        self.symmetry.group = value
+
+    @property
+    def hall_number(self):
+        """The hall number for the (periodic) configuration."""
+        return self.symmetry.hall_number
 
     @property
     def id(self):
@@ -365,32 +444,15 @@ class _Configuration(
         return sum(masses)
 
     @property
-    def spin_multiplicity(self):
-        """The spin_multiplicity of the system, 0, 1, 2, 3, ..."""
-        if self._spin_multiplicity == 0:
-            self.cursor.execute(
-                "SELECT spin_multiplicity FROM configuration WHERE id = ?", (self.id,)
-            )
-            multiplicity = self.cursor.fetchone()[0]
-            if multiplicity is None or multiplicity == 0:
-                n_electrons = sum(self.atoms.atomic_numbers) - self.charge
-                if n_electrons % 2 == 0:
-                    multiplicity = 1
-                else:
-                    multiplicity = 2
-                self.spin_multiplicity = multiplicity
-            else:
-                self._spin_multiplicity = multiplicity
-        return self._spin_multiplicity
+    def n_asymmetric_atoms(self) -> int:
+        """The number of symmetry-unique atoms.
 
-    @spin_multiplicity.setter
-    def spin_multiplicity(self, value):
-        self.cursor.execute(
-            "UPDATE configuration SET spin_multiplicity = ? WHERE id = ?",
-            (value, self.id),
-        )
-        self.db.commit()
-        self._spin_multiplicity = value
+        Returns
+        -------
+        int
+            Number of atoms
+        """
+        return self.atoms.n_asymmetric_atoms
 
     @property
     def n_atoms(self) -> int:
@@ -411,6 +473,8 @@ class _Configuration(
                 "SELECT name FROM configuration WHERE id = ?", (self.id,)
             )
             self._name = self.cursor.fetchone()[0]
+            if self._name is None:
+                self._name = ""
         return self._name
 
     @name.setter
@@ -422,6 +486,17 @@ class _Configuration(
         self.db.commit()
         self._name = value
 
+    @property
+    def n_asymmetric_bonds(self) -> int:
+        """The number of asymmetric bonds.
+
+        Returns
+        -------
+        int
+            Number of asymmetric bonds
+        """
+        return self.bonds.n_asymmetric_bonds
+
     @property
     def n_bonds(self) -> int:
         """The number of bonds.
@@ -465,6 +540,11 @@ class _Configuration(
         )
         self.db.commit()
 
+    @property
+    def n_symops(self):
+        """The number of symmetry operations in the group."""
+        return self.symmetry.n_symops
+
     @property
     def periodicity(self):
         """The periodicity of the system, 0, 1, 2 or 3"""
@@ -484,12 +564,75 @@ class _Configuration(
         )
         self.db.commit()
         self._periodicity = value
+        if value == 0:
+            self.symmetry.group = "C1"
+        else:
+            self.symmetry.group = "P 1"
 
     @property
     def properties(self):
         """The class to handle the properties for this configuration."""
         return _ConfigurationProperties(self)
 
+    @property
+    def spin_multiplicity(self):
+        """The spin_multiplicity of the system, 0, 1, 2, 3, ..."""
+        if self._spin_multiplicity == 0:
+            self.cursor.execute(
+                "SELECT spin_multiplicity FROM configuration WHERE id = ?", (self.id,)
+            )
+            multiplicity = self.cursor.fetchone()[0]
+            if multiplicity is None or multiplicity == 0:
+                n_electrons = sum(self.atoms.atomic_numbers) - self.charge
+                if n_electrons % 2 == 0:
+                    multiplicity = 1
+                else:
+                    multiplicity = 2
+                self.spin_multiplicity = multiplicity
+            else:
+                self._spin_multiplicity = multiplicity
+        return self._spin_multiplicity
+
+    @spin_multiplicity.setter
+    def spin_multiplicity(self, value):
+        self.cursor.execute(
+            "UPDATE configuration SET spin_multiplicity = ? WHERE id = ?",
+            (value, self.id),
+        )
+        self.db.commit()
+        self._spin_multiplicity = value
+
+    @property
+    def spin_state(self):
+        """Return the text spin state, lke 'triplet'.
+
+        Returns
+        -------
+        state : str
+            The spin state as text, e.g. singlet, doublet, etc.
+        """
+        if self.spin_multiplicity < len(spin_states):
+            state = spin_states[self.spin_multiplicity - 1]
+        else:
+            state = f"{self.spin_multiplicity}-let"
+        return state
+
+    @spin_state.setter
+    def spin_state(self, value):
+        tmp = value.lower()
+        if tmp in spin_states:
+            self.spin_multiplicity = spin_states.index(tmp) + 1
+            return
+
+        if value.endswith("-let"):
+            try:
+                multiplicity = int(value[:-5])
+            except ValueError:
+                raise ValueError(f"Unknown spin state '{value}'")
+            self.spin_multiplicity = multiplicity
+
+        raise ValueError(f"Unknown spin state '{value}'")
+
     @property
     def subsets(self):
         """The subsets"""
@@ -513,7 +656,9 @@ class _Configuration(
     @property
     def symmetry(self):
         """The periodic (unit) symmetry for this configuration."""
-        return _Symmetry(self)
+        if self._symmetry is None:
+            self._symmetry = _Symmetry(self)
+        return self._symmetry
 
     @property
     def symmetry_id(self):
@@ -533,6 +678,25 @@ class _Configuration(
                 self.db.commit()
         return self._symmetry_id
 
+    @property
+    def symmetry_matrices(self):
+        """The 4x4 matrices for the symmetry operations."""
+        return self.symmetry.symmetry_matrices
+
+    @property
+    def symops(self):
+        """The symmetry operators as shorthand strings."""
+        return self.symmetry.symops
+
+    @symops.setter
+    def symops(self, value):
+        self.symmetry.symops = value
+
+    @property
+    def symop_to_atom(self):
+        """List of list of symop #'s for creating symmetry atoms from asymmetric."""
+        return self.symmetry.symop_to_atom
+
     @property
     def system(self):
         """The system that we belong to."""
@@ -582,7 +746,113 @@ class _Configuration(
         # Delete the atoms
         self.atoms.delete("all")
 
-    def primitive_cell(self, symprec=1.0e-05, spg=False):
+    def get_symmetry_data(self, hall_number):
+        """Get the raw spglib symmetry data for the system given a Hall number"""
+        cell_in = (
+            self.cell.vectors(),
+            self.atoms.get_coordinates(fractionals=True),
+            self.atoms.atomic_numbers,
+        )
+        tmp = spglib.get_symmetry_dataset(cell_in, hall_number=hall_number)
+        return tmp
+
+    def lower_symmetry(self, other=None):
+        """Lower the symmetry to P1 or C1, optionally from another configuration.
+
+        Parameters
+        ----------
+        other : molsystem._Configuration = None
+            Another configuration to use as the source
+        """
+        if other is None:
+            other = self
+
+        periodicity = other.periodicity
+
+        if periodicity != 0:
+            cell_parameters = other.cell.parameters
+
+        # Get the atom and bond information for low symmetry
+        atom_data = other.atoms.get_as_dict()
+        del atom_data["id"]
+        bond_data = other.bonds.get_as_dict()
+        del bond_data["id"]
+
+        # It is not clear that it makes sense to handle velocities, so drop
+        if "vx" in atom_data:
+            del atom_data["vx"]
+            del atom_data["vy"]
+            del atom_data["vz"]
+
+        self.clear()
+        self.new_atomset()
+        self.new_bondset()
+        if periodicity != 0:
+            self.new_cell()
+            self.cell.parameters = cell_parameters
+        self.new_symmetry()
+        self.periodicity = periodicity
+
+        ids = self.atoms.append(**atom_data)
+
+        iatoms = bond_data["i"]
+        jatoms = bond_data["j"]
+        bond_data["i"] = [ids[i] for i in iatoms]
+        bond_data["j"] = [ids[j] for j in jatoms]
+
+        self.bonds.append(**bond_data)
+
+        # Finally, copy over the charge multiplicity, etc.
+        if self is not other:
+            self.charge = other.charge
+            self.spin_multiplicity = other.spin_multiplicity
+            self.n_active_electrons = other.n_active_electrons
+            self.n_active_orbitals = other.n_active_orbitals
+
+    def new_atomset(self):
+        """Setup a new, empty atomset for this configuration."""
+        self._atomset = self.system_db["atomset"].append(n=1)[0]
+        self.db.execute(
+            "UPDATE configuration SET atomset = ? WHERE id = ?",
+            (self._atomset, self.id),
+        )
+        self.db.commit()
+        return self._atomset
+
+    def new_bondset(self):
+        """Setup a new, empty bondset for this configuration."""
+        self._bondset = self.system_db["bondset"].append(n=1)[0]
+        self.db.execute(
+            "UPDATE configuration SET bondset = ? WHERE id = ?",
+            (self._bondset, self.id),
+        )
+        self.db.commit()
+        return self._bondset
+
+    def new_cell(self):
+        """Setup a new cell for this configuration."""
+        self._cell_id = self.system_db["cell"].append(n=1)[0]
+        sql = "UPDATE configuration SET cell = ? WHERE id = ?"
+        self.db.execute(sql, (self._cell_id, self.id))
+        self.db.commit()
+
+        return self.cell
+
+    def new_symmetry(self):
+        """Create a new symmetry object for this configuration."""
+        self._symmetry_id = None
+
+        table = _Table(self.system_db, "symmetry")
+        self._symmetry_id = table.append()[0]
+        self.cursor.execute(
+            "UPDATE configuration SET symmetry = ? WHERE id = ?",
+            (self._symmetry_id, self.id),
+        )
+        self.db.commit()
+
+        return self.symmetry
+
+    def primitive_cell(self, symprec=1.0e-05, spg=True):
         """Find the symmetry of periodic systems and transform to conventional cell.
 
         Parameters
@@ -613,7 +883,9 @@ class _Configuration(
         cell_in = (lattice_in, fractionals_in, atomic_numbers_in)
 
         # Need this to get the mapping to the primitive cell...uff!
-        tmp = spglib.get_symmetry_dataset(cell_in, symprec=symprec)
+        tmp = spglib.get_symmetry_dataset(
+            cell_in, symprec=symprec, hall_number=self.hall_number
+        )
         mapping_to_primitive = [*tmp["mapping_to_primitive"]]
         n_max = max(mapping_to_primitive)
         mapping_from_primitive = [None for i in range(n_max + 1)]
@@ -622,9 +894,7 @@ class _Configuration(
                 mapping_from_primitive[prim] = at
 
         if spg:
-            lattice, fractionals, atomic_numbers = spglib.standardize_cell(
-                cell_in, to_primitive=True, no_idealize=True, symprec=symprec
-            )
+            lattice, fractionals, atomic_numbers = spglib.find_primitive(cell_in)
         else:
             dataset = seekpath.get_path(cell_in, symprec=symprec)
             lattice = dataset["primitive_lattice"].tolist()
@@ -671,7 +941,7 @@ class _Configuration(
         else:
             raise NotImplementedError("Only affine transformations so far.")
 
-    def symmetrize(self, symprec=1.0e-05, angle_tolerance=None, spg=False):
+    def symmetrize(self, symprec=1.0e-05, angle_tolerance=None, spg=True):
         """Find the symmetry of periodic systems and transform to conventional cell.
 
         Parameters
@@ -694,7 +964,7 @@ class _Configuration(
             recommended approach!
 
         spg : bool = False
-            Whether to use ``spglib`` or ``seekpath`` (the default)
+            Whether to use ``spglib`` (default) or ``seekpath``
         """
         if self.periodicity == 3:
             lattice_in = self.cell.vectors()
@@ -718,10 +988,15 @@ class _Configuration(
             if spg:
                 if angle_tolerance is not None:
                     dataset = spglib.get_symmetry_dataset(
-                        cell_in, symprec=symprec, angle_tolerance=angle_tolerance
+                        cell_in,
+                        symprec=symprec,
+                        angle_tolerance=angle_tolerance,
+                        hall_number=self.hall_number,
                     )
                 else:
-                    dataset = spglib.get_symmetry_dataset(cell_in, symprec=symprec)
+                    dataset = spglib.get_symmetry_dataset(
+                        cell_in, symprec=symprec, hall_number=self.hall_number
+                    )
 
                 lattice = dataset["std_lattice"].tolist()
                 fractionals = dataset["std_positions"].tolist()
@@ -740,6 +1015,9 @@ class _Configuration(
                 atomic_numbers = dataset["conv_types"].tolist()
                 space_group = dataset["spacegroup_international"].strip("'")
 
+            print(f"{self.n_atoms}")
+            pprint.pprint(dataset)
+
             logger.debug("")
             logger.debug("Lattice after symmetrize")
             for vector in lattice:
@@ -765,6 +1043,7 @@ class _Configuration(
             self.coordinate_system = "fractional"
             self.atoms.append(atno=atomic_numbers, x=xs, y=ys, z=zs)
             self.symmetry.group = space_group
+            print(f"{self.n_atoms}")
 
     def update(
         self,
@@ -775,7 +1054,7 @@ class _Configuration(
         space_group=None,
         symprec=1.0e-05,
         angle_tolerance=None,
-        spg=False,
+        spg=True,
     ):
         """Update the system, checking symmetry.
 
@@ -814,24 +1093,33 @@ class _Configuration(
             recommended approach!
 
         spg : bool = False
-            Whether to use ``spglib`` or ``seekpath`` (the default)
+            Whether to use ``spglib`` (default) or ``seekpath``
         """
         text = ""
 
         cell_in = (lattice, coordinates, atomic_numbers)
 
         if spg:
+            print(f"{self.hall_number=}")
             if angle_tolerance is not None:
                 dataset = spglib.get_symmetry_dataset(
-                    cell_in, symprec=symprec, angle_tolerance=angle_tolerance
+                    cell_in,
+                    symprec=symprec,
+                    angle_tolerance=angle_tolerance,
+                    hall_number=self.hall_number,
                 )
             else:
-                dataset = spglib.get_symmetry_dataset(cell_in, symprec=symprec)
+                dataset = spglib.get_symmetry_dataset(
+                    cell_in, symprec=symprec, hall_number=self.hall_number
+                )
 
+            pprint.pprint(dataset)
             lattice_out = dataset["std_lattice"].tolist()
             fractionals_out = dataset["std_positions"].tolist()
             atomic_numbers_out = dataset["std_types"].tolist()
-            space_group_out = dataset["international"].strip("'")
+            space_group_out = self.symmetry.hall_to_spacegroup_name(
+                dataset["hall_number"]
+            )
         else:
             if angle_tolerance is not None:
                 dataset = seekpath.get_path(
@@ -856,20 +1144,38 @@ class _Configuration(
             )
         logger.debug(f"{space_group_out}")
 
+        print("Lattice")
+        for vector in lattice_out:
+            print(f"   {vector[0]:7.3f}   {vector[1]:7.3f}   {vector[2]:7.3f}")
+        print("")
+        print("Fractionals")
+        for vector, atno in zip(fractionals_out, atomic_numbers_out):
+            print(f"   {atno:2} {vector[0]:7.3f}   {vector[1]:7.3f}   {vector[2]:7.3f}")
+        print(f"{space_group_out}")
+
         if space_group != space_group_out:
             text = f"The space group changed from {space_group} to {space_group_out}."
             logger.debug(text)
+            raise RuntimeError(text)
 
         self.cell.from_vectors(lattice_out)
 
-        self.atoms.delete("all")
+        # self.atoms.delete("all")
 
-        xs = [f[0] for f in fractionals_out]
-        ys = [f[1] for f in fractionals_out]
-        zs = [f[2] for f in fractionals_out]
+        # xs = [f[0] for f in fractionals_out]
+        # ys = [f[1] for f in fractionals_out]
+        # zs = [f[2] for f in fractionals_out]
 
-        self.coordinate_system = "fractional"
-        self.atoms.append(atno=atomic_numbers_out, x=xs, y=ys, z=zs)
-        self.symmetry.group = space_group_out
+        # self.coordinate_system = "fractional"
+        # self.atoms.append(atno=atomic_numbers_out, x=xs, y=ys, z=zs)
+        # self.symmetry.group = space_group_out
+
+        print("Updating the coordinates to these")
+        pprint.pprint(fractionals_out)
+        self.atoms.set_coordinates(fractionals_out)
+
+        print("coordinates in system after update")
+        pprint.pprint(self.coordinates)
+        print("")
 
         return text
