@@ -38,7 +38,6 @@ def add_properties_from_file(path):
     that it is indeed the same, it can be replaced by the standard form, and also
     aliased to it for backwards compatibility.
     """
-    global standard_properties
     with open(path, newline="", encoding="utf-8-sig") as fd:
         data = csv.reader(fd)
         line = 0
@@ -87,7 +86,6 @@ class _Properties(object):
 
     @property
     def standard_properties(self):
-        global standard_properties
         return standard_properties
 
     @property
@@ -254,7 +252,7 @@ class _Properties(object):
         )
         self.db.execute("CREATE INDEX str_data_idx_system ON str_data(system)")
 
-        # Integer facts
+        # Array facts
         table = self.system_db["json_data"]
         table.add_attribute("id", coltype="int", pk=True)
         table.add_attribute("configuration", coltype="int", references="configuration")
@@ -317,149 +315,143 @@ class _Properties(object):
         self.cursor.execute("SELECT COUNT(*) FROM property WHERE name = ?", (name,))
         return self.cursor.fetchone()[0] != 0
 
-    def get(self, configuration_id, _property="all"):
-        """Get the given property value for the configuration.
+    def get(
+        self,
+        _id,
+        pattern="*",
+        match="glob",
+        is_system=False,
+        include_system_properties=False,
+        include_configuration_properties=False,
+        types=["float", "int", "str", "json"],
+    ):
+        """Get the property value(s)
 
         Parameters
         ----------
-        configuration_id : int
-            The id of the configuration.
-        _property : int or str, or "all"
-            The id or name of the property, or all properties if "all".
+        _id : int
+            The id of the configuration or system.
+        is_system : bool=False
+            Whether the ID is for a system, not a configuration.
+        pattern : str = "*"
+            The pattern of the property.
+        match : str="glob"
+            Whether to use exact, glob, or 'like' matching.
+        include_system_properties : bool=False
+            For a configuration, whether to include properties that are on the system.
+        include_configuration_properties : bool=False
+            For a system, whether to include properties that are on the configurations
+            of the system.
+        types : [str] = ["float", "int", "str", "json"]
+            The type of results to return.
 
         Returns
         -------
-        int, float, or str
-            The value of the property.
+        {str: {str: value}}
+            The matching property values.
         """
-        if _property == "all":
-            sql = "SELECT name, type, value"
-            sql += "  FROM property, float_data"
-            sql += " WHERE float_data.property = property.id AND configuration = ?"
-            sql += " UNION "
-            sql += "SELECT name, type, value"
-            sql += "  FROM property, int_data"
-            sql += " WHERE int_data.property = property.id AND configuration = ?"
-            sql += " UNION "
-            sql += "SELECT name, type, value"
-            sql += "  FROM property, str_data"
-            sql += " WHERE str_data.property = property.id AND configuration = ?"
-            sql += " UNION "
-            sql += "SELECT name, type, value"
-            sql += "  FROM property, json_data"
-            sql += " WHERE json_data.property = property.id AND configuration = ?"
-
-            self.cursor.execute(
-                sql,
-                (
-                    configuration_id,
-                    configuration_id,
-                    configuration_id,
-                    configuration_id,
-                ),
-            )
-
-            result = {}
-            for row in self.cursor:
-                name, _type, value = row
-                if _type == "float":
-                    result[name] = float(value)
-                elif _type == "int":
-                    result[name] = int(value)
-                elif _type == "json":
-                    result[name] = json.loads(value)
-                else:
-                    result[name] = value
-            return result
+        if match == "glob":
+            op = "GLOB"
+        elif match == "like":
+            op = "LIKE"
+        elif match == "exact":
+            op = "="
         else:
-            if isinstance(_property, str):
-                if self.exists(_property):
-                    pid = self.id(_property)
-                else:
-                    raise ValueError(f"Property '{_property}' does not exist.")
-            else:
-                pid = _property
+            raise ValueError(f"Unknown match type '{match}'.")
 
-            ptype = self.type(pid)
-            sql = (
-                f"SELECT value FROM {ptype}_data"
-                "  WHERE configuration = ? AND property = ?"
-            )
-            self.cursor.execute(sql, (configuration_id, pid))
-            result = self.cursor.fetchone()
-            if result is None:
-                raise ValueError(
-                    f"Property {_property} does not exist for configuration "
-                    f"{configuration_id}"
+        sql = ""
+        args = []
+
+        if is_system:
+            # Handle properties of systems
+            if include_configuration_properties:
+                for _type in types:
+                    if len(args) > 1:
+                        sql += "        UNION\n"
+                    sql += (
+                        "       SELECT name, type, value, system, configuration\n"
+                        f"         FROM property, {_type}_data\n"
+                        "        WHERE system = ?\n"
+                        f"          AND {_type}_data.property = property.id\n"
+                        f"          AND name {op} ?\n"
+                    )
+                    args.append(_id)
+                    args.append(pattern)
+            else:
+                for _type in types:
+                    if len(args) > 1:
+                        sql += "        UNION\n"
+                    sql += (
+                        "       SELECT name, type, value, system, configuration\n"
+                        f"         FROM property, {_type}_data\n"
+                        "        WHERE system = ?\n"
+                        "          AND configuration is Null\n"
+                        f"          AND {_type}_data.property = property.id\n"
+                        f"          AND name {op} ?\n"
+                    )
+                    args.append(_id)
+                    args.append(pattern)
+        else:
+            for _type in types:
+                if len(args) > 1:
+                    sql += "        UNION\n"
+                sql += (
+                    "       SELECT name, type, value, system, configuration\n"
+                    f"         FROM property, {_type}_data\n"
+                    "        WHERE configuration = ?\n"
+                    f"          AND {_type}_data.property = property.id\n"
+                    f"          AND name {op} ?\n"
                 )
-            if ptype == "json":
-                return json.loads(result[0])
+                args.append(_id)
+                args.append(pattern)
+            if include_system_properties:
+                self.cursor.execute(
+                    "SELECT system FROM configuration WHERE id = ?", (_id,)
+                )
+                system_id = self.cursor.fetchone()[0]
+                for _type in types:
+                    if len(args) > 1:
+                        sql += "        UNION\n"
+                    sql += (
+                        "       SELECT name, type, value, system, configuration\n"
+                        f"         FROM property, {_type}_data\n"
+                        "        WHERE system = ?\n"
+                        "          AND configuration is Null\n"
+                        f"          AND {_type}_data.property = property.id\n"
+                        f"          AND name {op} ?\n"
+                    )
+                    args.append(system_id)
+                    args.append(pattern)
+
+        self.cursor.execute(sql, args)
+
+        result = {}
+        for row in self.cursor:
+            name, _type, value, sid, cid = row
+            result[name] = {
+                "sid": sid,
+                "cid": cid,
+            }
+            if _type == "float":
+                try:
+                    result[name]["value"] = float(value)
+                except Exception as e:
+                    logger.warning(f"Error with value of property '{name}': {str(e)}")
+                    del result[name]
+            elif _type == "int":
+                try:
+                    result[name]["value"] = int(value)
+                except Exception as e:
+                    logger.warning(f"Error with value of property '{name}': {str(e)}")
+                    del result[name]
+            elif _type == "json":
+                try:
+                    result[name]["value"] = json.loads(value)
+                except Exception as e:
+                    logger.warning(f"Error with value of property '{name}': {str(e)}")
+                    del result[name]
             else:
-                return result[0]
-
-    def get_for_system(self, system_id, _property="all"):
-        """Get the given property value(s) for the system.
-
-        Parameters
-        ----------
-        system_id : int
-            The id of the system.
-        _property : int or str of "all"
-            The id or name of the property, or "all" for all properties.
-
-        Returns
-        -------
-        [int, float, or str]
-            The value(s) of the property.
-        """
-        if _property == "all":
-            sql = "SELECT name, type, value"
-            sql += "  FROM property, float_data"
-            sql += " WHERE float_data.property = property.id AND system = ?"
-            sql += " UNION "
-            sql += "SELECT name, type, value"
-            sql += "  FROM property, int_data"
-            sql += " WHERE int_data.property = property.id AND system = ?"
-            sql += " UNION "
-            sql += "SELECT name, type, value"
-            sql += "  FROM property, str_data"
-            sql += " WHERE str_data.property = property.id AND system = ?"
-            sql += " UNION "
-            sql += "SELECT name, type, value"
-            sql += "  FROM property, json_data"
-            sql += " WHERE json_data.property = property.id AND system = ?"
-
-            self.cursor.execute(sql, (system_id, system_id, system_id, system_id))
-
-            result = {}
-            for row in self.cursor:
-                name, _type, value = row
-                if _type == "float":
-                    result[name] = float(value)
-                elif _type == "int":
-                    result[name] = int(value)
-                elif _type == "json":
-                    result[name] = json.loads(value)
-                else:
-                    result[name] = value
-        else:
-            if isinstance(_property, str):
-                if self.exists(_property):
-                    pid = self.id(_property)
-                else:
-                    raise ValueError(f"Property '{_property}' does not exist.")
-            else:
-                pid = _property
-            ptype = self.type(pid)
-
-            sql = f"SELECT value FROM {ptype}_data WHERE system = ? AND property = ?"
-            result = []
-            for row in self.db.execute(sql, (system_id, pid)):
-                if ptype == "json":
-                    result.append(json.loads(row[0]))
-                else:
-                    result.append(row[0])
-
+                result[name]["value"] = value
         return result
 
     def id(self, name):
@@ -487,6 +479,106 @@ class _Properties(object):
         self.cursor.execute("SELECT name FROM property")
         return [row[0] for row in self.cursor.fetchall()]
 
+    def list(
+        self,
+        _id,
+        pattern="*",
+        match="glob",
+        is_system=False,
+        include_system_properties=False,
+        include_configuration_properties=False,
+        as_ids=False,
+        types=["float", "int", "str", "json"],
+    ):
+        """Get the list of matching properties for the configuration.
+
+        Parameters
+        ----------
+        _id : int
+            The id of the configuration.
+        is_system : bool=False
+            Whether the ID is for a system, not a configuration.
+        pattern : str = "*"
+            The pattern of the property.
+        match : str="glob"
+            Whether to use exact, glob, or 'like' matching.
+        include_system_properties : bool=False
+            For a configuration, whether to include properties that are on the system.
+        include_configuration_properties : bool=False
+            For a system, whether to include properties that are on the configurations
+            of the system.
+        as_ids : bool=False
+            Whether to return the ids rather than names
+        types : [str] = ["float", "int", "str", "json"]
+            The type of results to return.
+
+        Returns
+        -------
+        [str] or [int]
+            The matching properties.
+        """
+        if match == "glob":
+            op = "GLOB"
+        elif match == "like":
+            op = "LIKE"
+        elif match == "exact":
+            op = "="
+        else:
+            raise ValueError(f"Unknown match type '{match}'.")
+
+        if as_ids:
+            sql = "SELECT id FROM property\n"
+        else:
+            sql = "SELECT name FROM property\n"
+        sql += f" WHERE name {op} ?\n" "    AND id IN (\n"
+
+        args = [pattern]
+
+        if is_system:
+            # Handle properties of systems
+            if include_configuration_properties:
+                for _type in types:
+                    if len(args) > 1:
+                        sql += "        UNION\n"
+                        f"       SELECT property FROM {_type}_data WHERE system = ?\n"
+                    args.append(_id)
+            else:
+                for _type in types:
+                    if len(args) > 1:
+                        sql += "        UNION\n"
+                    sql += (
+                        "       SELECT property\n"
+                        f"         FROM {_type}_data\n"
+                        "        WHERE system = ? AND configuration is Null\n"
+                    )
+                    args.append(_id)
+        else:
+            for _type in types:
+                if len(args) > 1:
+                    sql += "        UNION\n"
+                sql += f"  SELECT property FROM {_type}_data WHERE configuration = ?\n"
+                args.append(_id)
+            if include_system_properties:
+                self.cursor.execute(
+                    "SELECT system FROM configuration WHERE id = ?", (_id,)
+                )
+                system_id = self.cursor.fetchone()[0]
+                for _type in types:
+                    if len(args) > 1:
+                        sql += "        UNION\n"
+                    sql += (
+                        "       SELECT property\n"
+                        f"         FROM {_type}_data\n"
+                        "        WHERE configuration is Null AND system = ?\n"
+                    )
+                    args.append(system_id)
+
+        sql += "    )"
+
+        self.cursor.execute(sql, args)
+
+        return [row[0] for row in self.cursor]
+
     def metadata(self, _property):
         """The metadata for a property
 
@@ -552,17 +644,21 @@ class _Properties(object):
         "Obsolete routine kept for compatibility"
         return self.type(_property)
 
-    def put(self, configuration_id, _property, value):
-        """Store the given property value for the configuration.
+    def put(self, _id, _property, value, is_system=False):
+        """Store the given property value for the configuration or system
+
+        If the property value already exists, overwrite it in situ.
 
         Parameters
         ----------
-        configuration_id : int
+        _id : int
             The id of the configuration.
         _property : int or str
             The id or name of the property.
         value : int, float, or str
             The value to store.
+        is_system : bool=False
+            Whether the ID is for a system, not a configuration.
         """
         # Get the property id and type
         if isinstance(_property, str):
@@ -579,50 +675,52 @@ class _Properties(object):
         if ptype == "json":
             value = json.dumps(value, separators=(",", ":"))
 
-        # Get the system id
-        self.cursor.execute(
-            "SELECT system FROM configuration WHERE id = ?", (configuration_id,)
-        )
-        system_id = self.cursor.fetchone()[0]
-
-        sql = (
-            f"INSERT INTO {ptype}_data (configuration, system, property, value)"
-            "      VALUES(?, ?, ?, ?)"
-        )
-        self.db.execute(sql, (configuration_id, system_id, pid, value))
-
-    def put_for_system(self, system_id, _property, value):
-        """Store the given property value for the system.
-
-        Parameters
-        ----------
-        system_id : int
-            The id of the system.
-        _property : int or str
-            The id or name of the property.
-        value : int, float, or str
-            The value to store.
-        """
-        # Get the property id and type
-        if isinstance(_property, str):
-            if not self.exists(_property):
-                if _property in self.standard_properties:
-                    self.add(_property)
-                else:
-                    raise ValueError(f"Property '{_property}' does not exist.")
-            pid = self.id(_property)
+        if is_system:
+            # Check if a value already exists
+            sql = (
+                f"SELECT COUNT(*) FROM {ptype}_data WHERE system = ? AND "
+                "configuration is Null AND property = ?"
+            )
+            self.cursor.execute(sql, (_id, pid))
+            if self.cursor.fetchone()[0] == 0:
+                # Not there so add
+                sql = (
+                    f"INSERT INTO {ptype}_data (system, property, value)"
+                    "      VALUES(?, ?, ?)"
+                )
+                self.db.execute(sql, (_id, pid, value))
+            else:
+                # update in place
+                sql = (
+                    f"UPDATE {ptype}_data SET value = ? WHERE system = ? AND "
+                    "configuration is Null AND property = ?"
+                )
+                self.db.execute(sql, (value, _id, pid))
         else:
-            pid = _property
-        ptype = self.type(pid)
+            # Get the system id
+            self.cursor.execute("SELECT system FROM configuration WHERE id = ?", (_id,))
+            sid = self.cursor.fetchone()[0]
 
-        if ptype == "json":
-            value = json.dumps(value, separators=(",", ":"))
-
-        sql = (
-            f"INSERT INTO {ptype}_data (system, property, value)"
-            "      VALUES(?, ?, ?)"
-        )
-        self.db.execute(sql, (system_id, pid, value))
+            # Check if a value already exists
+            sql = (
+                f"SELECT COUNT(*) FROM {ptype}_data WHERE system = ? AND "
+                "configuration = ? AND property = ?"
+            )
+            self.cursor.execute(sql, (sid, _id, pid))
+            if self.cursor.fetchone()[0] == 0:
+                # Not there so add
+                sql = (
+                    f"INSERT INTO {ptype}_data (configuration, system, property, value)"
+                    "      VALUES(?, ?, ?, ?)"
+                )
+                self.db.execute(sql, (_id, sid, pid, value))
+            else:
+                # update in place
+                sql = (
+                    f"UPDATE {ptype}_data SET value = ? WHERE system = ? AND "
+                    "configuration = ? AND property = ?"
+                )
+                self.db.execute(sql, (value, sid, _id, pid))
 
     def query(self, *args, what=["configuration"]):
         """Find configurations that match the query defined by the args.
