@@ -1,7 +1,6 @@
 # -*- coding: utf-8 -*-
 
-"""A dictionary-like object for holding a system
-"""
+"""A dictionary-like object for holding a system"""
 
 from collections.abc import MutableMapping
 import logging
@@ -93,7 +92,6 @@ class _System(CIFMixin, MutableMapping):
         self._system_db = system_db
         self._id = _id
         self.logger = logger
-        self._current_configuration_id = None  # The current configuration
         self._checkpoints = []
         self._items = {}
 
@@ -118,7 +116,7 @@ class _System(CIFMixin, MutableMapping):
         table : Table or Atom, Template, Templateatoms, ...
         """
         if key not in self._items:
-            self._items[key] = _Table(self, key)
+            self._items[key] = _Table(self.system_db, key)
         return self._items[key]
 
     def __setitem__(self, key, value):
@@ -192,7 +190,7 @@ class _System(CIFMixin, MutableMapping):
         # Check the tables in both systems
         result = True
         for table in in_common:
-            if _Table(self, table) != _Table(other, table):
+            if _Table(self.system_db, table) != _Table(other.system_db, table):
                 result = False
                 break
 
@@ -207,11 +205,15 @@ class _System(CIFMixin, MutableMapping):
         """The configuration object for the current configuration."""
         if self.n_configurations == 0:
             return None
-        if self._current_configuration_id is None:
-            self._current_configuration_id = self.configuration_ids[-1]
-        return _Configuration(
-            _id=self._current_configuration_id, system_db=self.system_db
+        self.cursor.execute(
+            "SELECT default_configuration FROM system WHERE id = ?",
+            (self.id,),
         )
+        result = self.cursor.fetchone()[0]
+        if result is None:
+            result = self.configuration_ids[-1]
+            self.configuration = result
+        return _Configuration(_id=result, system_db=self.system_db)
 
     @configuration.setter
     def configuration(self, value):
@@ -223,7 +225,11 @@ class _System(CIFMixin, MutableMapping):
 
         if value not in self.configuration_ids:
             raise KeyError(f"configuration '{value}' does not exist.")
-        self._current_configuration_id = value
+        self.db.execute(
+            "UPDATE system SET default_configuration = ? WHERE id = ?",
+            (value, self.id),
+        )
+        self.db.commit()
 
     @property
     def configuration_names(self):
@@ -276,6 +282,8 @@ class _System(CIFMixin, MutableMapping):
         if result is None:
             return None
         else:
+            if result[0] is None:
+                return ""
             return result[0]
 
     @name.setter
@@ -429,10 +437,13 @@ class _System(CIFMixin, MutableMapping):
             system=self.id, periodicity=periodicity, **kwargs
         )[0]
         configuration = _Configuration(_id=cid, system_db=self.system_db)
+        if symmetry is None:
+            # Make sure that symmetry is set up...
+            configuration.periodicity = periodicity
 
         # If the atomset was given, need to create dummy coordinates
         if atomset is not None:
-            n_atoms = configuration.n_atoms
+            n_atoms = configuration.n_asymmetric_atoms
             data = {"configuration": configuration.id}
             data["atom"] = configuration.atoms.ids
             data["x"] = [0.0] * n_atoms
@@ -448,7 +459,7 @@ class _System(CIFMixin, MutableMapping):
             configuration.cell.parameters = cell_parameters
 
         if make_current:
-            self._current_configuration_id = cid
+            self.configuration = cid
 
         return configuration
 
@@ -495,25 +506,25 @@ class _System(CIFMixin, MutableMapping):
 
         # Need the contents of the tables. See if they are in the same
         # database or if we need to attach the other database temporarily.
-        name = self.nickname
-        other_name = other.nickname
-        detach = False
-        if name != other_name and not self.is_attached(other_name):
-            # Attach the other system in order to do comparisons.
-            self.attach(other)
-            detach = True
+        # name = self.nickname
+        # other_name = other.nickname
+        # detach = False
+        # if name != other_name and not self.is_attached(other_name):
+        #     # Attach the other system in order to do comparisons.
+        #     self.attach(other)
+        #     detach = True
 
         # Check the tables in both systems
         for table in in_common:
-            table1 = _Table(self, table)
-            table2 = _Table(other, table)
+            table1 = _Table(self.system_db, table)
+            table2 = _Table(other.system_db, table)
             tmp = table1.diff(table2)
-            if len(tmp) > 0:
+            if tmp is not None:
                 result[f"table '{table}' diffs"] = tmp
 
         # Detach the other database if needed
-        if detach:
-            self.detach(other)
+        # if detach:
+        #     self.detach(other)
 
         return result
 
