@@ -1,7 +1,6 @@
 # -*- coding: utf-8 -*-
 
-"""A dictionary-like object for holding a system
-"""
+"""A dictionary-like object for holding a system"""
 
 import collections.abc
 import logging
@@ -129,10 +128,6 @@ class SystemDB(CIFMixin, collections.abc.MutableMapping):
     symmetry
         Information about point or space group symmetry.
 
-    symmetryoperation
-        The detailed symmetry operations for the point or space
-        groups.
-
     cell
         The information about the periodicity of a configuration, if
         needed.
@@ -274,7 +269,7 @@ class SystemDB(CIFMixin, collections.abc.MutableMapping):
     def __eq__(self, other):
         """Return a boolean if this object is equal to another"""
         # LGTM doesn't like using 'is', but this is equivalent.
-        if self.id() == other.id():
+        if self is other:
             return True
 
         tables = set(self.list())
@@ -312,6 +307,16 @@ class SystemDB(CIFMixin, collections.abc.MutableMapping):
 
         return result
 
+    @property
+    def configurations(self):
+        """The list of configuration objects."""
+        return [_Configuration(_id, self) for _id in self.configuration_ids]
+
+    @property
+    def configuration_ids(self):
+        """The list of configuration ids."""
+        return [row[0] for row in self.db.execute("SELECT id FROM configuration")]
+
     @property
     def cursor(self):
         """A database cursor."""
@@ -345,9 +350,9 @@ class SystemDB(CIFMixin, collections.abc.MutableMapping):
         self._filename = value
         if self._filename is not None:
             if self._filename[0:5] == "file:":
-                self._db = sqlite3.connect(self._filename, uri=True)
+                self._db = sqlite3.connect(self._filename, uri=True, timeout=10.0)
             else:
-                self._db = sqlite3.connect(self._filename)
+                self._db = sqlite3.connect(self._filename, timeout=10.0)
             self._db.row_factory = sqlite3.Row
             self._db.execute("PRAGMA foreign_keys = ON")
             self._db.execute("PRAGMA synchronous = normal")
@@ -584,7 +589,7 @@ class SystemDB(CIFMixin, collections.abc.MutableMapping):
         """Differences between this system and another."""
         result = {}
 
-        if self.id() == other.id():
+        if self is other:
             return result
 
         tables = set(self.list())
@@ -612,12 +617,15 @@ class SystemDB(CIFMixin, collections.abc.MutableMapping):
         # Check the tables in both systems
         for table in in_common:
             tmp = self[table].diff(other[table])
-            if len(tmp) > 0:
+            if tmp is not None:
                 result[f"table '{table}' diffs"] = tmp
 
         # Detach the other database if needed
         if detach:
-            self.detach(other)
+            try:
+                self.detach(other)
+            except sqlite3.OperationalError:
+                pass
 
         return result
 
@@ -676,6 +684,50 @@ class SystemDB(CIFMixin, collections.abc.MutableMapping):
         """
         return _Configuration(_id=cid, system_db=self)
 
+    def get_configurations(self, pattern="*"):
+        """Return the configurations matching the glob pattern.
+
+        Parameters
+        ----------
+        pattern : list or str = "*"
+            The glob-style pattern for matching the configuration names
+
+        Returns
+        -------
+        [_configuration]
+            The requested configuration ids
+        """
+        return [
+            _Configuration(_id, self) for _id in self.get_configuration_ids(pattern)
+        ]
+
+    def get_configuration_ids(self, pattern="*"):
+        """Return the configuration ids matching the glob pattern.
+
+        Parameters
+        ----------
+        pattern : list or str = "*"
+            The glob-style pattern for matching the configuration names
+
+        Returns
+        -------
+        [int]
+            The requested configuration ids
+        """
+        if pattern == "*":
+            return [row[0] for row in self.db.execute("SELECT id FROM configuration")]
+
+        if isinstance(pattern, list):
+            sql = "\n UNION \n".join(
+                ["SELECT id FROM configuration WHERE name GLOB ?"] * len(pattern)
+            )
+            params = pattern
+        else:
+            sql = "SELECT id FROM configuration WHERE name GLOB ?"
+            params = (pattern,)
+
+        return [row[0] for row in self.db.execute(sql, params)]
+
     def get_system(self, id_or_name):
         """Get the specified system object.
 
@@ -707,6 +759,47 @@ class SystemDB(CIFMixin, collections.abc.MutableMapping):
                 id_or_name = systems[0]
         return _System(self, id_or_name)
 
+    def get_systems(self, pattern="*"):
+        """Return the systems matching the glob pattern.
+
+        Parameters
+        ----------
+        pattern : list or str = "*"
+            The glob-style pattern for matching the system names
+
+        Returns
+        -------
+        [_system]
+            The requested system ids
+        """
+        return [_System(self, _id) for _id in self.get_system_ids(pattern)]
+
+    def get_system_ids(self, pattern="*"):
+        """Return the system ids matching the glob pattern.
+
+        Parameters
+        ----------
+        pattern : list or str = "*"
+            The glob-style pattern for matching the system names
+
+        Returns
+        -------
+        [int]
+            The requested system ids
+        """
+        if pattern == "*":
+            return [row[0] for row in self.db.execute("SELECT id FROM system")]
+
+        if isinstance(pattern, list):
+            sql = "\n UNION \n".join(
+                ["SELECT id FROM system WHERE name GLOB ?"] * len(pattern)
+            )
+            params = pattern
+        else:
+            sql = "SELECT id FROM system WHERE name GLOB ?"
+            params = (pattern,)
+        return [row[0] for row in self.db.execute(sql, params)]
+
     def is_attached(self, other):
         """Return whether another system is attached to this one.
 
@@ -770,11 +863,8 @@ class SystemDB(CIFMixin, collections.abc.MutableMapping):
             # Symmetry information
             table = self["symmetry"]
             table.add_attribute("id", coltype="int", pk=True)
-            table.add_attribute("group", coltype="str", default="")
-
-            table = self["symmetryoperation"]
-            table.add_attribute("symmetry", coltype="int", references="symmetry")
-            table.add_attribute("symop", coltype="str")
+            table.add_attribute("group", coltype="str", default="C1")
+            table.add_attribute("symops", coltype="str", default="x,y,z")
 
             # Periodic cell information
             table = self["cell"]
@@ -790,6 +880,7 @@ class SystemDB(CIFMixin, collections.abc.MutableMapping):
             table = self["system"]
             table.add_attribute("id", coltype="int", pk=True)
             table.add_attribute("name", coltype="str", notnull=True, default="default")
+            table.add_attribute("default_configuration", coltype="int")
 
             # The atoms, and the sets of atoms
             table = self["atom"]
@@ -813,10 +904,8 @@ class SystemDB(CIFMixin, collections.abc.MutableMapping):
             table.add_attribute("i", coltype="int", references="atom")
             table.add_attribute("j", coltype="int", references="atom")
             table.add_attribute("bondorder", coltype="int", default=1)
-            # Only need for infinite networked systems. Think about later!
-            # table.add_attribute("offset1", coltype="int", default=0)
-            # table.add_attribute("offset2", coltype="int", default=0)
-            # table.add_attribute("offset3", coltype="int", default=0)
+            table.add_attribute("symop1", coltype="str", default=".")
+            table.add_attribute("symop2", coltype="str", default=".")
 
             table = self["bondset"]
             table.add_attribute("id", coltype="int", pk=True)
@@ -866,6 +955,39 @@ class SystemDB(CIFMixin, collections.abc.MutableMapping):
                 "CREATE INDEX idx_coordinates_atom_configuration "
                 "    ON coordinates(atom, configuration)"
             )
+
+            # Keep the velocities separate from coordinates to keep the size down
+            table = self["velocities"]
+            table.add_attribute(
+                "configuration", coltype="int", references="configuration"
+            )
+            table.add_attribute("atom", coltype="int", references="atom")
+            table.add_attribute("vx", coltype="float")
+            table.add_attribute("vy", coltype="float")
+            table.add_attribute("vz", coltype="float")
+            self.db.execute(
+                "CREATE INDEX 'idx_velocities_atom' ON velocities (\"atom\")"
+            )
+            self.db.execute(
+                "CREATE INDEX idx_velocities_atom_configuration "
+                "    ON velocities(atom, configuration)"
+            )
+
+            # Keep the gradients separate from coordinates to keep the size down
+            table = self["gradients"]
+            table.add_attribute(
+                "configuration", coltype="int", references="configuration"
+            )
+            table.add_attribute("atom", coltype="int", references="atom")
+            table.add_attribute("gx", coltype="float")
+            table.add_attribute("gy", coltype="float")
+            table.add_attribute("gz", coltype="float")
+            self.db.execute("CREATE INDEX 'idx_gradients_atom' ON gradients (\"atom\")")
+            self.db.execute(
+                "CREATE INDEX idx_gradients_atom_configuration "
+                "    ON gradients(atom, configuration)"
+            )
+
             # The definition of the subsets -- templates
             table = self["template"]
             table.add_attribute("id", coltype="int", pk=True)
@@ -876,8 +998,8 @@ class SystemDB(CIFMixin, collections.abc.MutableMapping):
                 "configuration", coltype="int", references="configuration"
             )
             self.db.execute(
-                "CREATE UNIQUE INDEX 'idx_template_name_type'"
-                '    ON template ("name", "type")'
+                "CREATE UNIQUE INDEX 'idx_template_name_category'"
+                "    ON template (name, category)"
             )
 
             # The subsets
@@ -914,7 +1036,7 @@ class SystemDB(CIFMixin, collections.abc.MutableMapping):
             table.add_attribute("system", coltype="int", references="system")
             self.db.execute(
                 "CREATE INDEX 'idx_system_collection_system' "
-                'ON atomset_atom ("system_collection", "system")'
+                'ON system_collection_system ("system_collection", "system")'
             )
 
             # Collections of configurations
@@ -933,7 +1055,8 @@ class SystemDB(CIFMixin, collections.abc.MutableMapping):
             )
             self.db.execute(
                 "CREATE INDEX 'idx_configuration_collection_configuration' "
-                'ON atomset_atom ("configuration_collection", "configuration")'
+                "ON configuration_collection_configuration "
+                '("configuration_collection", "configuration")'
             )
 
             #####################################################
