@@ -3,7 +3,7 @@
 
 import pytest  # noqa: F401
 
-"""Tests for handling Molfiles."""
+"""Tests for handling crystallographic information files (CIF)."""
 
 cif_cu = """\
 # CIF file 
@@ -298,7 +298,8 @@ MOL1 C H3 SING
 MOL1 C C2 SING
 MOL1 C2 O DOUB
 MOL1 C2 O2 SING
-MOL1 O2 H4 SING"""
+MOL1 O2 H4 SING
+"""
 
     text = AceticAcid.to_mmcif_text()
 
@@ -309,25 +310,24 @@ MOL1 O2 H4 SING"""
 
 
 def test_to_cif_text(vanadium):
-    """Write a manually created system to mmcif"""
+    """Write a manually created system to cif"""
     correct = """\
 # Generated by MolSSI SEAMM
-data_V
-_symmetry_space_group_name_H-M   'P 1'
+data_SEAMM:default/BCCVanadium
+_space_group_name_H-M_full   'P 1'
 _cell_length_a   3.03
 _cell_length_b   3.03
 _cell_length_c   3.03
 _cell_angle_alpha   90.0
 _cell_angle_beta    90.0
 _cell_angle_gamma   90.0
-_symmetry_Int_Tables_number   1
 _cell_volume   27.818126999999997
 _cell_formula_units_Z   2
 loop_
  _symmetry_equiv_pos_site_id
  _symmetry_equiv_pos_as_xyz
-  1  'x, y, z'
-_chemical_formula_structural   V
+  1  x,y,z
+_chemical_formula_structural   'V'
 _chemical_formula_sum   'V2'
 loop_
  _atom_site_type_symbol
@@ -337,8 +337,9 @@ loop_
  _atom_site_fract_y
  _atom_site_fract_z
  _atom_site_occupancy
-V V  1  0.000 0.000 0.000  1
-V V2  1  0.500 0.500 0.500  1"""
+  V V1  1  0.000 0.000 0.000  1
+  V V2  1  0.500 0.500 0.500  1
+"""
 
     text = vanadium.to_cif_text()
 
@@ -353,4 +354,141 @@ def test_from_cif_text(configuration):
 
     configuration.from_cif_text(cif_cu)
 
+    # configuration.symmetry._expand()
+
+    assert configuration.atoms.n_asymmetric_atoms == 1
     assert configuration.atoms.n_atoms == 4
+    assert configuration.atoms.asymmetric_symbols == ["Cu"]
+    assert configuration.atoms.symbols == ["Cu"] * 4
+
+
+def test_bonds(polyethylene):
+    """Test the bonds in a polyethylene crystal."""
+    correct = (
+        "1.0577 1.0577 1.0577 1.0577 "
+        "1.0548 1.0548 1.0548 1.0548 "
+        "1.5383 1.5383 1.5383 1.5383"
+    )
+
+    # polyethylene.symmetry.loglevel = "DEBUG"
+    # polyethylene.bonds.loglevel = "DEBUG"
+
+    assert polyethylene.n_atoms == 12
+    assert polyethylene.n_bonds == 12
+
+    lengths = " ".join([f"{r:.4f}" for r in polyethylene.bonds.get_lengths()])
+    if lengths != correct:
+        print(lengths)
+
+    assert lengths == correct
+
+
+def test_benzene_bonds(benzene):
+    """Test the bonds in a benzene crystal."""
+    correct = (
+        "1.3788 1.3788 1.3788 1.3788 1.3788 1.3788 1.3788 1.3788 "
+        "1.3820 1.3820 1.3820 1.3820 1.3820 1.3820 1.3820 1.3820 "
+        "0.9300 0.9300 0.9300 0.9300 0.9300 0.9300 0.9300 0.9300 "
+        "1.3761 1.3761 1.3761 1.3761 1.3761 1.3761 1.3761 1.3761 "
+        "0.9306 0.9306 0.9306 0.9306 0.9306 0.9306 0.9306 0.9306 "
+        "0.9296 0.9296 0.9296 0.9296 0.9296 0.9296 0.9296 0.9296"
+    )
+
+    # benzene.symmetry.loglevel = "DEBUG"
+    # benzene.bonds.loglevel = "DEBUG"
+
+    assert benzene.n_atoms == 48
+    assert benzene.n_bonds == 48
+
+    lengths = " ".join([f"{r:.4f}" for r in benzene.bonds.get_lengths()])
+    if lengths != correct:
+        print(lengths)
+
+    assert lengths == correct
+
+
+def test_benzene_to_cif(benzene):
+    """Test producing a cif file for benzene."""
+    bond_lengths = (
+        "1.3788 1.3788 1.3788 1.3788 1.3788 1.3788 1.3788 1.3788 "
+        "1.3820 1.3820 1.3820 1.3820 1.3820 1.3820 1.3820 1.3820 "
+        "0.9300 0.9300 0.9300 0.9300 0.9300 0.9300 0.9300 0.9300 "
+        "1.3761 1.3761 1.3761 1.3761 1.3761 1.3761 1.3761 1.3761 "
+        "0.9306 0.9306 0.9306 0.9306 0.9306 0.9306 0.9306 0.9306 "
+        "0.9296 0.9296 0.9296 0.9296 0.9296 0.9296 0.9296 0.9296"
+    )
+    correct = """\
+# Generated by MolSSI SEAMM
+data_SEAMM:default/default
+_space_group_name_H-M_full       'P 2_1/b 2_1/c 2_1/a'
+_symmetry_space_group_name_Hall '-P 2ac 2ab'
+_space_group_IT_number          61
+_cell_length_a   6.914
+_cell_length_b   7.476
+_cell_length_c   9.563
+_cell_angle_alpha   90.0
+_cell_angle_beta    90.0
+_cell_angle_gamma   90.0
+_cell_volume   494.30251903199996
+_cell_formula_units_Z   24
+loop_
+ _symmetry_equiv_pos_site_id
+ _symmetry_equiv_pos_as_xyz
+  1 x,y,z
+  2 1/2-x,-y,1/2+z
+  3 1/2+x,1/2-y,-z
+  4 -x,1/2+y,1/2-z
+  5 -x,-y,-z
+  6 1/2+x,y,1/2-z
+  7 1/2-x,1/2+y,z
+  8 x,1/2-y,1/2+z
+_chemical_formula_structural   'C H'
+_chemical_formula_sum   'C24 H24'
+loop_
+ _atom_site_type_symbol
+ _atom_site_label
+ _atom_site_symmetry_multiplicity
+ _atom_site_fract_x
+ _atom_site_fract_y
+ _atom_site_fract_z
+ _atom_site_occupancy
+  C C1  1  0.130 0.576 0.408  1
+  H H1  1  0.217 0.627 0.346  1
+  C C2  1  0.123 0.633 0.545  1
+  H H2  1  0.207 0.723 0.576  1
+  C C3  1  0.006 0.443 0.363  1
+  H H3  1  0.009 0.405 0.270  1
+loop_
+_geom_bond_atom_site_label_1
+_geom_bond_atom_site_label_2
+_geom_bond_distance
+_geom_bond_site_symmetry_1
+_geom_bond_site_symmetry_2
+  C1 C2 1.3788 . .
+  C1 C3 1.3820 . .
+  C1 H1 0.9300 . .
+  C2 C3 1.3761 . 5_566
+  C2 H2 0.9306 . .
+  C3 H3 0.9296 . .
+"""
+
+    # benzene.symmetry.loglevel = "DEBUG"
+    # benzene.bonds.loglevel = "DEBUG"
+
+    text = benzene.to_cif_text()
+    if text != correct:
+        print(text)
+    assert text == correct
+
+    system = benzene.system
+    configuration = system.create_configuration()
+    configuration.from_cif_text(text)
+
+    assert benzene.n_atoms == 48
+    assert benzene.n_bonds == 48
+
+    lengths = " ".join([f"{r:.4f}" for r in benzene.bonds.get_lengths()])
+    if lengths != bond_lengths:
+        print(lengths)
+
+    assert lengths == bond_lengths
