@@ -9,6 +9,8 @@ import platform
 
 import pytest  # noqa: F401
 
+from molsystem import openbabel_version
+
 # H-Asp-Arg-Val-Tyr-Ile-His-Pro-Phe-OH
 SMILES = (
     "[NH2][C@@H](CC(=O)O)C(=O)"
@@ -365,6 +367,81 @@ known_input_formats["Linux"] = (
 )
 known_input_formats["Windows"] = known_input_formats["Darwin"]
 
+copper_sdf = """SEAMM=default/FCC Copper
+ OpenBabel03252515063D
+
+  4  0  0  0  0  0  0  0  0  0999 V2000
+    0.0000    0.0000    0.0000 Cu  0  0  0  0  0 15  0  0  0  0  0  0
+    1.8075    1.8075    0.0000 Cu  0  0  0  0  0  0  0  0  0  0  0  0
+    1.8075    0.0000    1.8075 Cu  0  0  0  0  0  0  0  0  0  0  0  0
+    0.0000    1.8075    1.8075 Cu  0  0  0  0  0  0  0  0  0  0  0  0
+M  RAD  1   1   1
+M  END
+>  <SEAMM|net charge|int|>
+0
+
+>  <SEAMM|spin multiplicity|int|>
+1
+
+>  <SEAMM|XYZ|json|>
+[
+    [0, 0, 0],
+    [1.807455, 1.807455, 0],
+    [1.807455, 0, 1.807455],
+    [0, 1.807455, 1.807455]
+]
+
+>  <SEAMM|cell|json|>
+[3.61491, 3.61491, 3.61491, 90, 90, 90]
+
+>  <SEAMM|system name|str|>
+default
+
+>  <SEAMM|configuration name|str|>
+FCC Copper
+
+$$$$"""
+
+copper_sdf_2 = """SEAMM=default/FCC Copper
+ OpenBabel03252519453D
+
+  4  0  0  0  0  0  0  0  0  0999 V2000
+    0.0000    0.0000    0.0000 Cu  0  0  0  0  0 15  0  0  0  0  0  0
+    1.8075    1.8075    0.0000 Cu  0  0  0  0  0  0  0  0  0  0  0  0
+    1.8075    3.6149    1.8075 Cu  0  0  0  0  0  0  0  0  0  0  0  0
+    3.6149    1.8075    1.8075 Cu  0  0  0  0  0  0  0  0  0  0  0  0
+M  RAD  1   1   1
+M  END
+>  <SEAMM|net charge|int|>
+0
+
+>  <SEAMM|spin multiplicity|int|>
+1
+
+>  <SEAMM|XYZ|json|>
+[
+    [0, 0, 0],
+    [1.807455, 1.807455, 0],
+    [1.807455, 0, 1.807455],
+    [0, 1.807455, 1.807455]
+]
+
+>  <SEAMM|cell|json|>
+[3.61491, 3.61491, 3.61491, 90, 90, 90]
+
+>  <SEAMM|system name|str|>
+default
+
+>  <SEAMM|configuration name|str|>
+FCC Copper
+
+$$$$"""
+
+
+def test_version():
+    """Test the version number of Open Babel."""
+    openbabel_version()
+
 
 def test_substructure(CH3COOH_3H2O):
     """Test the finding substructures in the configuration."""
@@ -429,7 +506,7 @@ def test_substructure_ordering(disordered):
     assert result == answer2
 
     # With hydrogens
-    smiles = templates[0].to_smiles(hydrogens=True)
+    smiles = templates[0].to_smiles(hydrogens=True, flavor="openbabel")
     if smiles != answer3:
         pprint.pprint(smiles)
     assert smiles == answer3
@@ -458,7 +535,7 @@ def test_all_residue_search(configuration):
         "PHE_LL": [(64, 65, 66, 67, 68, 69, 70, 71, 72, 73, 74, 75)],
     }
 
-    configuration.from_smiles(SMILES)
+    configuration.from_smiles(SMILES, reorient=False, flavor="openbabel")
 
     for name, sc in sidechains.items():
         smarts = f"[NH1X3][C@@H]({sc})[CX3]=[OX1]"
@@ -491,22 +568,56 @@ def test_all_residue_search(configuration):
             assert result == c_terminal[name]
 
 
-def test_to_OBMol(configuration):
+def test_to_OBMol(Acetate):
     """Test creating an OBMol object from a structure."""
-    mol = configuration.to_OBMol()
+    correct = {
+        "SEAMM|XYZ|json|": (
+            "[\n"
+            "    [1.0797, 0.0181, -0.0184],\n"
+            "    [0.5782, 3.1376, 0.2813],\n"
+            "    [0.7209, -0.6736, -0.7859],\n"
+            "    [0.7052, -0.3143, 0.9529],\n"
+            "    [0.5713, 1.3899, -0.3161],\n"
+            "    [-0.1323, 1.7142, -1.2568],\n"
+            "    [0.9757, 2.297, 0.5919]\n"
+            "]"
+        ),
+        "SEAMM|configuration name|str|": "acetate",
+        "SEAMM|float property|float|kcal/mol": 3.14,
+        "SEAMM|int property|int|": 2,
+        "SEAMM|net charge|int|": -1,
+        "SEAMM|spin multiplicity|int|": 1,
+        "SEAMM|str property|str|": "Hi!",
+        "SEAMM|system name|str|": "acetate",
+    }
+
+    mol = Acetate.to_OBMol(properties="*")
 
     bondorder_list = []
     for bond in openbabel.OBMolBondIter(mol):
         bondorder_list.append(bond.GetBondOrder())
 
     atno_list = []
-    for atno in openbabel.OBMolAtomIter(mol):
-        atno_list.append(mol.GetAtmoicNum())
+    for atom in openbabel.OBMolAtomIter(mol):
+        atno_list.append(atom.GetAtomicNum())
 
-    assert configuration.atoms.atomic_numbers == atno_list
-    assert configuration.bonds.bondorders == bondorder_list
-    # assert configuration.n_atoms == mol.NumAtoms()
-    # assert configuration.n_bonds == mol.NumBonds()
+    assert Acetate.atoms.atomic_numbers == atno_list
+    assert Acetate.bonds.bondorders == bondorder_list
+
+    data = {}
+    for item in mol.GetData():
+        value = item.GetValue()
+        try:
+            value = int(value)
+        except Exception:
+            try:
+                value = float(value)
+            except Exception:
+                pass
+        data[item.GetAttribute()] = value
+    if data != correct:
+        pprint.pprint(data)
+    assert data == correct
 
 
 def test_from_OBMol(configuration):
@@ -541,3 +652,47 @@ def test_input_formats():
         print(system)
         pprint.pprint(formats)
     assert formats == known_input_formats[system]
+
+
+def test_copper_to_sdf(copper):
+    """Write a manually created configuration to molfile"""
+    saved = copper.to_sdf_text()
+    tmp = saved.splitlines()
+    del tmp[1]
+    text = "\n".join(tmp)
+
+    correct = copper_sdf.splitlines()
+    del correct[1]
+    correct = "\n".join(correct)
+
+    if text != correct:
+        print(saved)
+    assert text == correct
+
+
+def test_copper_from_sdf(copper):
+    """Read an sdf file for periodic system"""
+
+    system = copper.system.system_db.create_system()
+    configuration = system.create_configuration()
+    sysname, confname = configuration.from_sdf_text(copper_sdf, properties="")
+    configuration.name = confname
+    configuration.system.name = sysname
+
+    result = configuration.system.diff(copper.system)
+    if result != {}:
+        pprint.pprint(result)
+    assert result == {}
+
+    saved = configuration.to_sdf_text()
+    tmp = saved.splitlines()
+    del tmp[1]
+    text = "\n".join(tmp)
+
+    correct = copper_sdf_2.splitlines()
+    del correct[1]
+    correct = "\n".join(correct)
+
+    if text != correct:
+        print(saved)
+    assert text == correct
